// 
// File megastat.h

/////////////////////////////////
// Used for nonBorland compilers
#define M_LOG2E  1.44269504088896340736  // 1 / log (2)
#define M_LOG10E 0.434294481903251827651 // 1 / log (10)
#define M_PI    3.14159265358979323846  //  ""
#define M_SQRT2  41421356237309504880  //    2
#define M_LN2     0.693147180559945309417

#define SQRT_2       1.4142135
#define SQRT_M_PI    1.7724538                //    ""
#define SQRT_M_PI_2  2.5066282                //    "2 * "

////////////////////////////////////////////////////////
// User defined function for NewtonCotes family methods
typedef double (*userFunc) (double);

//////////////////////////////////////////////////
// User defined functions for optimization methods
typedef double (*userFuncRight) (double*,int);
typedef void (*userFuncGrad)(double*,double*,int,double,
userFuncRight);
typedef void (*userFuncHesse)(double*,double*,int,double,
userFuncRight);

////////////////////////////////////////////////////////
// User defined function for experimental design methods
typedef void (*userFuncMany) (double,double*,int,double*);

/////////////
// Prototypes
double Abbe (double*,int);
double AbbeDistribution (int, double);
double AbsMaximumOf (double*,int);
double ApproximationWilks (double,int,int,int);
double Arcctg (double);
double Arth (double);
double Bayes (double*,int,int,int,int*,int*,double*);
double Beta (double,double);
double BetaDistribution (double,double,double);
double BetaS (double,double);
double Bravais (double*,double*,int);
double CanonicalCorrelation (double,double);
double ChiSquare (double*,double*,int);
double ChiSquareDistribution (double,double);
double ChiSquareEmpire (double*,double*,int,double);
double ChiSquareIntegral (double,double);
double CityBlock (double*,double*,int);
double CochraneGDistributionApprox (double,double,double);
double CochraneGDistributionExact (double,double,double);
double Combination (int,int);
double Concord (double*,int,int);
//double Confidence interval (int,double,double,int);
double Cov (double*,double*,int);
double CriticalW (int,double);
double DannDistribution95 (int);
double DannDistribution95c (int);
double DannDistribution99 (int);
double DannDistribution99c (int);
double Det (double*,int);
double DuncanDistribution (double,int,int);
double Entropie (double*,int);
double Euclid (double*,double*,int);
double EuclidNorma (double*,int);
double EuclidNormaTri (double*,int);
double EuclidNormaVector (double*,int);
double EuclidNormaWithoutDiag (double*,int);
double Factorial (int);
double FDistribution (double,double,double);
double Fechner (double*,double*,int);
double FeedforwardError (double*,int);
double FisherIrwin (int,int,int,int);
double Gamma (double);
double GammaE (double);
double GammaI (double);
double GammaS (double);
double Geary (double*,int);
double GearyDistribution (int, double);
double GTest (double*,double*,int);
double GTestDistribution (int,double);
double Hemming (double*,double*,int);
double Horner (double*,int,double);
double InfiniteNorma (double*,int);
double InverseAbbeDistribution (int, double);
double InverseBetaDistribution (double,double,double);
double InverseBetaDistributionBig (double,double,double);
double InverseChiSquareDistribution (double,double);
double InverseChiSquareDistributionBig (double,double);
double InverseFDistribution (double,double,double);
double InverseGearyDistribution (int, double);
double InverseGTestDistribution (int,double);
double InverseNormalDistribution (double);
double InversePagurovaDistribution (int,int,double,double);
double InverseSerialDistribution (int,int,int);
double InverseTDistribution (double,double);
double InverseXDistribution (double,double,double);
double Jaccard (double*,double*,int);
double KolmogorovDistribution (double);
double KolmogorovOneSided (double*,int);
double KolmogorovTwoSided (double*,int);
double Krauth (double*,double*,double*,int);
double Krauth (double*,int);
double Kullback (double*,double*,int);
double Lagrange (double,double*,double*,int);
double LambdaDistribution (double);
double LogFactorial (int);
double Mahalanobis (double*,double*,double*,int);
double MannWhitneyDistribution (int,int,double);
double MaxEntropie (int);
double MaximumOf (double*,int);
double MaximumOf (double*,int,int*);
double MaximumOf (double,double);
double MinimumOf (double*,int);
double MinimumOf (double*,int,int*);
double MinimumOf (double,double);
double Minkovski (double*,double*,int,int);
double Mode (double*,double*,int);
double Moment (int,double*,int,double*,int);
double NormalDistribution (double);
double OmegaSquare (double*,double*,int);
double OmegaSquareDistribution (double);
double Organization (double*,int);
double Pearson (double*,double*,int);
double PowerOf (double,int);
double Quality (int*,int,int*,int*);
double Rank (double*,double*,int);
double Range (double*,int);
double RangeNormal(double,int);
double RangeNormalDensity (double,double,int);
double RangeNormalIntegral (double,int);
double RangeStudent(double,int,int);
double RangeStudentDensity (double,double,int,int);
double RankKendall (double*,double*,int);
double Rao (double*,double*,int);
double RelEntropie (double,double);
double ShapiroWilk (double*,int);
double Sign (double);
double SignNull (double);
double SignTest (double*,double*,int);
double Simpson (double,double,int,userFunc);
double SmartRound (double x);
double Smirnov (double*,double*,int);
double SmirnovExact (double*,double*,int,int);
double Sokal (double*,double*,int);
double Stirling (int);
double StudentPairedTest (double*,double*,int);
double Sup (double*,double*,int);
double Surplus (double,double);
double TDistribution (double,double);
double Trace (double*,int);
double Trapezium (double,double,int,userFunc);
double UDistribution (int,int,double);
double VectorSum (double*,int);
double VectorSumSQ (double*,int);
double WilcoxonTDistribution (int,double);
double Wilks (double*,int,int,double);
double WTDistribution (int,double);
double Yule (double*,double*,int);
int AnsariBradleyTest (double*,double*,int,int,double*);
int Bartlett (double*,int*,int,double*);
int Biserial (double*,double*,int,double*);
int Biserial (double*,int,double*);
int BrandtSnedecor (double*,int,int,double*);
//int        Canonical Correlation Analysis (double*,double*,int,int,int,double*,double*,double*,double*,double*);
//int Canonical Discriminant Analysis (double*,int,int,double*,double*,double*,int*,int,int*,double*,double*);
int CDAClass (double*,int,int,int,double*,int,double*,int*);
int Choleski (double*,double*,int);
int Chuprov (double*,int,int,double*,double*);
int CochraneG (double*,int,int,double*);
int CochraneQ (double*,int,int,double*);
int Combination (double*,int,int,double,double*);
int ConditionNumber (double*,int,double*);
int ConjugateGradient (double*,int,double,int*,double,
userFuncGrad,userFuncRight);
int Dann (double*,int*,int,double*);
int BFGS (double*,int,double,int*,double,userFuncGrad,
userFuncRight);
int Derive (double,double*,int,double*,double*,userFuncMany);
int Eigen (double*,double*,int,double,int*);
int ExponentFunction (double*,double*,int,double*,double*);
int ExponentialFunction (double*,double*,int,double*);
int ExpPowerFunction (double*,double*,int,double*,double*);
int FDAClass (double*,int,int,int,double*,double*,int*);
int FisherDiscriminantAnalysis (double*,int,int,int*,int,int*,double*,double*);
int FrancisKublanovskaya (double*,int,double,int*);
int Friedman (double*,int,int,double*);
int FTest (double*,double*,int,int,double*);
int Gauss (double*,double*,int,int,double);
int GompertzFunction (double*,double*,int,double*,double*);
int Gower (double*,int,int,int*,double*);
int Gram (double*,int);
int GramSchmidt (double*,double*,int,int);
int Hotelling (double*,double*,int,int,int,double*);
int Householder (double*,int,double*,double*);
int HyperFunction (double*,double*,int,double*,double*);
int InverseMatrix (double*,double*,int);
int InversePolynomial (double*,double*,int,int,int,double*);
int Jacobi (double*,int,double,int*,double*);
int JonckheereTerpstra (double*,int*,int,double*,double*);
int KellyWood (double*,double*,int,double*);
int Kendall (double*,double*,int,double*);
int King (double*,int,int,int,int*,int*,double*);
int KruskalWallis (double*,int*,int,double*);
int LDAClass (double*,int,int,int,double*,double*,double*,
double*,int*);
int Likeness (double*,double*,double*,int,int,int);
int LikenessForCorrelation (double*,double*,int,int,int);
int LinearDiscriminantAnalysis (double*,int,int,int*,int,int*,
double*,double*,double*);
int LinearRegression (double*,double*,double*,int,int);
int LinearRegression0 (double*,double*,double*,int,int);
int LogarithmFunction (double*,double*,int,double*,double*,
double*);
int LogisticFunction (double*,double*,int,double*,double*,
double*);
int MannWhitneyTest (double*,double*,int,int,double*,double*);
int MaximumLikelihood (double*,int,double*,double*,int,int*);
int MaximumOf (int*,int);
int MaximumOf (int,int);
int MaximumPath (double*,int,int*,int*,double*);
int McQueen (double*,int,int,int,int,int*,double*,double*,int*);
int Median (double*,int,double*,double*);
int Median (double*,int,double*,int,double*,double*);
int MedianDistribution (int,int,double);
int MinimumOf (int*,int);
int MinimumOf (int,int);
int MultyFactor (double*,int,int,double*,double*);
int Near (double*,int,int,int,int*,int*,double*);
int NearLink (double*,int,int*,int*,double*);
int NelderMead (double*,int,double,int*,double,double,double,
double,userFuncRight);
int Newton (double*,double*,int,int*,double);
int Newton (double*,int,double,int*,double,userFuncHesse,
userFuncGrad,userFuncRight);
int NumberOfMinus (double*,int);
int OneWayAnalysis (double*,int*,int,double*);
int OrtoPlan (double*,int,int,int);
int Page (double*,int,int,double*,double*);
int Pagurova (double*,double*,int,int,double*,double*);
int Parity (int);
int PlusDef (double*,int);
int Point (double*,double*,int,double*);
int PolynomCoeff (double*,int,double*);
int Polynomial (double*,double*,int,int,int,double*);
int PowerFunction (double*,double*,int,double*,double*);
int PrincipalFactor (double*,int,double*,int*);
int PseudoInverseMatrix (double*,double*,int,int);
int QDistribution (int,int,double,int*);
int QRDecomposition (double*,double*,double*,int,int);
int RandomCriterion (double*,double*,int,double*);
int RandomCriterionIndependent (double*,double*,int,int,double*);
int RangeOfVar (double,double,double,double*);
int RangeOfVarDistribution (double,double,double*,double*);
int Reduction (double*,double*,int,int);
int Rings (double*,int,int*,int*,int*);
int Rosenbaum (double*,int,double*,int);
int Sarkadi (double*,int,int,double*,double*);
int Scheffe (double*,int*,int,double*);
int ScheffePair (double*,int,int,double*,double*);
int Sign (int);
int SignDistribution (int,double);
int SimpleIteration (double*,double*,double*,int,double,int*);
int SingularValueDecomposition (double*,int,int,double*,
double*,double*);
int Solves (double*,double*,int,int);
int Spearman (double*,double*,int,double*);
int SpecialVectorSum (int*,int);
int Student (double*,double*,int,int,double*,int*);
int Sturgesss (int);
int Thurstone (double*,int,double*,int);
int Torgerson (double*,int,double*,double*,int*);
int Tou (double*,int,int,int,double*);
int TwoDegree (int);
int VanDerWaerdenTest (double*,double*,int,int,double*);
int Varimax (double*,int,int);
int VectorSum (int*,int);
int WaldWolfowitz (double*,int,double*,int,double*);
int Ward (double*,int,int,int*,int*,double*);
int Welch (double*,double*,int,int,double*,int*);
int WilcoxonSignedTest (double*,double*,int,double*,double*);
int WilcoxonTest (double*,double*,int,int,double*,double*);
void Allocate (double*,int,double*,double*,int);
void AllocateCommon (double*,double*,double*,double*,
double*,int,int,int);
void ArrayToArray (double*,double*,int);
void CoeffOfVariance (double*,int,double*,double*);
void ColInterchange (double*,int,int,int);
void ColMultiplyNumber (double*,int,int,double);
void CombinationAdd (double*,int,int,double*,double,int*,
double*,int);
void Compare (double*,double*,int,int,int*,int*);
//void Confidence interval (double,int,double,double*,double*);
//void Confidence interval (int,double,double,int,double*,double*);
//void Confidence interval (int,double,double,int,double*,double*);
void Feedforward (double*,double*,double*);
void FillUp (double*,int,double);
void FillUp (int*,int,int);
void FillUpCol (double*,int,int,double);
void FillUpDiag (double*,int,double);
void FillUpRow (double*,int,int,int,double);
void Fourier (double*,int,int,double*,double*);
void FrequencyNormal (double*,double*,int,double,double);
void FullOrtoPlan (signed char*,int);
void Gerschgorin (double*,int,double*,double*);
void GetMinMax (double* xmin,double* xmax);
void Gini (double*,int,double*);
void Givens (double*,int,double*,int);
void GradC (double*,double*,int,double,userFuncRight);
void GradL (double*,double*,int,double,userFuncRight);
void GradR (double*,double*,int,double,userFuncRight);
void HesseC (double*,double*,int,double,userFuncRight);
void Hessenberg (double*,int);
void HesseR (double*,double*,int,double,userFuncRight);
void Kurtosis (double*,int,double*,double*);
void MatrixDivideNumber (double*,int,double);
void MatrixMinusDiag (double*,double*,int);
void MatrixMinusMatrix (double*,double*,double*,int);
void MatrixMinusMatrix (double*,double*,int);
void MatrixMultiplyMatrix  (double*,double*,double*,int,int,int);
void MatrixMultiplyMatrixT (double*,double*,double*,int,int,int);
void MatrixMultiplyNumber (double*,int,double);
void MatrixMultiplyVector (double*,double*,double*,int);
void MatrixPlusDiag (double*,double*,int);
void MatrixPlusMatrix (double*,double*,double*,int);
void MatrixPlusMatrix (double*,double*,int);
void MatrixT (double*,double*,int,int);
void MatrixTMultiplyMatrix (double*,double*,double*,int,int,int);
void MatrixUnit (double*,int);
void NormalizeMatrix (double*,int,int,double*);
void NumberInterchange (int*,int*);
void OptimalPlan (int,int*,int*);
void PercentOfVariance (double*,int,double*,double*,int);
void PolynomCoeffMinor (double*,int,double*);
void RankCommon (double*,double*,double*,double*,int,int);
void Reverse (double*,int);
void RowInterchange (double*,int,int,int,int);
void RowMultiplyNumber (double*,int,int,int,double);
void SampleMean (double*,int,double*,double*);
void ScalarMatrix (double*,double*,int);
void SerialDistribution (int,int,double,int*,int*);
void Sharp (double*,int,double*,double*);
void Skewness (double*,int,double*,double*);
void SortArrayCommon (double*,double*,int);
void SortArrayDown (double*,int,int*,int);
void SortArrayDown (int*,int,int*,int);
void SortArrayUp (double*,int,int*,int);
void SortArrayUp (int*,int,int*,int);
void UnSortArray (double*,double*,int,int*);
void Variance (double*,int,double*,double*);


//       
 double VectorSum(double* a, int n)
 {
   if((n==0)||(a==NULL)) return 0;
  
   double sum=0;

   for(int i=0; i<n; i++)
   {
     sum=sum + a[i];
   }

   return sum;

 }

//     
double Cov(double* a, int n)
 {
   if((n < 2)||(a==NULL)) return 0;  

   int i=0;
   double sum=0,sums=0, sum2=0, nsums2;
   
    for( i=0; i<n; i++)
   {
     sum = sum + a[i];
   } 
   sums=sum/n;            //   
   nsums2=n*sums*sums;    


   for( i=0; i<n; i++)
   {
     sum2=sum2 + a[i]*a[i];  //    
   }
  
  // 
  double dispersia = (sum2-nsums2)/(n-1); 

  return dispersia;

 }

 #include <iomanip.h>
#include <math.h>



// 
void ArrayToArray (double x[],double y[],int n)
//
//     .
// :
//   -  - ,
//  y -  - ,
//  n -   .
//  :
//  .
//
{
 for (register int i = 0; i < n; i++)
   y[i] = x[i];
}

///////////////////////////////////////
void FillUp (double a[],int n,double p)
//
//     .
// :
//  a -   n,
//  p - .
//  :
//  .
//
{
 for (register int i = 0; i < n; i++)
   a[i] = p;
}
/////////////////////////////////////////////



void FillUp (int a[],int n,int p)
//
//      .
// :
//  a -   n,
//  p - .
//  :
//  .
//
{
 for (register int i = 0; i < n; i++)
   a[i] = p;
}

//////////////////////////////////////////////////////////
// 
int Gauss (double b[],double a[],int row,int column,double eps)
//
//      
//       .
// :
//       b -     ,
//           ,   - 
//              ,
//       a -    row, ,
//     row -   -    a  b 
//             a,
//  column -    b,
//     eps -     .
//  :
//   0    ,
//  -2   .
//
{
 int i,j,k,    // 
     js,ji,ij,
     im,mi,mj,
     is,       //     
     ik,       //   
     in,       //    
     jn;       //    

 double guide, //    
        save;  // 

 ////////////////////////////////////////////
 //   -  
 for (k = 0, ik = -row; k < row; k++)
   {
    ik += row;
    is = ik + k;

    //////////////////////////
    //   
    for (i = k, ij = is, in = k, jn = k, guide = a[is];
         i < row; i++)
      for (j = k, im = ij++; j < row; j++)
        {
         if (fabs (a[im]) > fabs (guide))
          {
           guide = a[im];
in = i;
           jn = j;
          }
         im += row;
        }

    /////////////////////////
    //   
    if (fabs (guide) < eps)
      return -2;

    /////////////////////
    //  
    for (i = k, im = is, ij = ik + in; i < row; i++)
      {
       save = a[ij];
       a[ij] = a[im];
       a[im] = save / guide;
       ij += row;
       im += row;
      }
    for (i = 0, ij = in, im = k; i < column; i++)
      {
       save = b[ij];
       b[ij] = b[im];
       b[im] = save / guide;
       ij += row;
       im += row;
      }

    ////////////////////////
    //  
    for (i = 0, im = ik - 1, ij = row * jn - 1; i < row; i++)
      {
       save = a[++ij];
       a[ij] = a[++im];
       a[im] = save;
      }
    //////////////////////////////////////////
    //    
    ji = k + 1;
    for (i = ji, im = is, mi = is + row; i < row; i++)
      {
       for (j = ji, mj = mi, ij = ++im + row, save = a[im];
            j < row; j++)
         {
          a[ij] = a[ij] - a[mj] * save;
          mj += row;
          ij += row;
         }
       for (j = 0, ij = i, mj = k; j < column; j++)
         {
          b[ij] = b[ij] - b[mj] * save;
          ij += row;
          mj += row;
         }
      }

    ////////////////////////////////////////////////////////
    //      
    //     
    if (k == 0)
for (i = 0; i < row; i++)
        a[i] = (double) i;  //   
    save = a[jn];           //   
    a[jn] = a[k];
    a[k] = save;
   }
 
  /////////////////////////////////////////
 //   -  
 for (i = row - 1, im = row * row - 1; i > 0; i--)
   for (js = 0, --im, in = i - 1, mi = 0; js < column; js++)
     {
      for (j = row - 1, ij = im, save = 0; j >= i; j--)
        {
         save += b[j + mi] * a[ij];
         ij -= row;
        }
      ji = in + mi;
      b[ji] = b[ji] - save;
      mi += row;
     }

 /////////////////////////////////
 //   
 for (i = 0, j = 0; i < row; i++)
   {
    in = (int) (a[j]);
    if (in == j)
      j++;        //      
    else
      {
       for (js = 0, ij = in, ji = j; js < column; js++)
         {
          save = b[ij];
          b[ij] = b[ji];
          b[ji] = save;
          ij += row;
          ji += row;
         }
       save = a[in];
       a[in] = a[j];
       a[j] = save;
      }
   }
 return 0;
}




int Polynomial (double y[],double x[],int u,int m,int n,double c[])
//
//    .
// :
//   y -    n,
//   x -    n,
//   u -   ,
//   m -  ,
//   c -     m - u + 1.
//  :
//   0    ,
//  -1   ,
//  -2   .
// : m <= n.
//
{
 register int i,j,k,l; // 

 int Code = 0,         //  
     mu = m - u + 1;   //  

 double *z,            //   
        *b;            //   

 if ((z = new double[mu * mu]) == 0)
   return -1;
 if ((b = new double[mu]) == 0)
   {
    delete [] z;
    return -1;
   }

 //////////////////////////////////////////////////////
 //     
 FillUp (z,mu * mu,0);
 FillUp (b,mu,0);

 ////////////////////////////////////////
 //    
 for (k = u; k <= m; k++)
   {
    for (i = 0; i < n; i++)
      {
       for (j = u, l = k - u; j <= m; j++)
         {
          z[l] += pow (x[i],j + k);
          l += m - u + 1;
         }
       b[k - u] += y[i] * pow (x[i],k);
      }
   }

 /////////////////////////////////////
 //    
 if (!(Code = Gauss (b,z,mu,1,0.000001)))
   ArrayToArray (b,c,mu);

 delete [] z; delete [] b;

 return Code;
}


